package main.core.data.dt;

import main.core.data.ICalculationResult;
import main.core.data.IGrid;
import main.core.data.IWufiData;
import main.core.data.dt.primitives.MyPoint;
import translation.Translation;

import java.io.IOException;
import java.util.*;

/*
 * @author Pankratz
 * Date: 07.12.2005
 * Time: 10:38:39
 */

/**
 * $Id: CalculationResult.java,v 1.2 2006/11/03 15:19:07 fpankratz Exp $
 */
public class CalculationResult implements ICalculationResult {
    private int step;
    private IWufiData wd;
    private Enum resultOption;
    private IGrid grid;

    private Date startDate;
    private Date endDate;
    private Date currentDate;

    private int startStep;
    private int endStep;

    private List<MyPoint> positions;
    private List<Map<Enum,float[][]>> results;

    public CalculationResult(IWufiData wd, Enum resultOption,Date startDate,Date endDate,int startStep, int endStep, Set<MyPoint> positions,IGrid grid){
        this.wd = wd;
        this.resultOption = resultOption;
        this.startDate = startDate;
        this.endDate = endDate;
        this.grid = grid;

        step = startStep-1;
        this.startStep = startStep;
        this.endStep = endStep;

        this.positions = new ArrayList<MyPoint>(positions);
    }

    private void readStep() throws IOException {
        results = new ArrayList<Map<Enum,float[][]>>();
        Set<Date> date = new HashSet<Date>();
        for(int i = 0;i<positions.size();i++){
            MyPoint position = positions.get(i);
            Map<Enum,float[][]> temp = new HashMap<Enum,float[][]>();
            Date l = wd.getResultElements(resultOption, step, position, temp);
            date.add(l);
            results.add(temp);
        }        
        if(date.size() != 1){
            throw new IOException(Translation.translate("calculationresult.error.time"));
        }
        Iterator<Date> it = date.iterator();
        currentDate = it.next();
    }

    public boolean reset() throws IOException {
        step = startStep;
        readStep();
        return true;
    }

    public boolean seek(int step) throws IOException {
        if(step < startStep || step > endStep) throw new IllegalArgumentException("Invalid step");
        this.step = step;
        readStep();
        return true;
    }

    public boolean nextStep() throws IOException {
        if(step > endStep) return false;
        step++;
        readStep();
        return true;
    }

    public boolean previousStep() throws IOException {
        if(step <= startStep) return false;
        step--;
        readStep();
        return true;
    }

    public List<Map<Enum,float[][]>> getResults() {
        return results;
    }

    public int getStep() {
        return step;
    }

    public int getStepCount() {
        return endStep - startStep+1;
    }

    public Date getStartDate() {
        return startDate;
    }

    public Date getEndDate() {
        return endDate;
    }

    public Date getCurrentDate() {
        return currentDate;
    }

    public IGrid getGrid() {
        return grid;
    }

    public List<MyPoint> getPositions() {
        return positions;
    }
}
