package main.core.data.dt;

import main.core.data.IGrid;
import main.core.data.IGridElement;
import main.core.data.IMaterial;

import java.util.List;

/*
 * @author Pankratz
 * Date: 30.11.2005
 * Time: 15:57:45
 */

/**
 * $Id: Grid.java,v 1.1 2006/09/22 13:25:25 fpankratz Exp $
 */
public class Grid implements IGrid{
    private List<Float> layoutx;
    private List<Float> layouty;
    private float[] relativeGridLayoutX;
    private float[] relativeGridLayoutY;

    private IGridElement[][] gridelements;
    private double[][] factor;
    private double volume;

    public Grid(List<Float> layoutx, List<Float> layouty, IMaterial[][] materials) {
        this.layoutx = layoutx;
        this.layouty = layouty;
        this.gridelements = new IGridElement[layoutx.size()][layouty.size()];
        factor = new double[layoutx.size()][layouty.size()];
        volume = 0;

        float xold = 0;
        for(int i=0;i<layoutx.size();i++){
            float fx = layoutx.get(i);
            float yold = 0;
            for(int j=0;j<layouty.size();j++){
                float fy = layouty.get(j);
                IGridElement ge;
                if(materials[i][j] == null){
                    ge = new GridElement(null,fx-xold,fy-yold);
                } else {
                    ge = new GridElement(materials[i][j],fx-xold,fy-yold);
                    volume += ge.getVolume();
                }
                gridelements[i][j] = ge;
                yold = fy;
            }
            xold = fx;
        }
      
        for(int i=0;i<gridelements.length;i++){
            for(int j=0;j<gridelements[i].length;j++){
                factor[i][j] = gridelements[i][j].getVolume();
            }
        }

        relativeGridLayoutX = new float[layoutx.size()];
        relativeGridLayoutX[0] = layoutx.get(0);
        for(int i = 1;i<layoutx.size();i++){
            relativeGridLayoutX[i] = layoutx.get(i).floatValue() - layoutx.get(i-1).floatValue();
        }

        relativeGridLayoutY = new float[layouty.size()];
        relativeGridLayoutY[0] = layouty.get(0);
        for(int i = 1;i<layouty.size();i++){
            relativeGridLayoutY[i] = layouty.get(i).floatValue() - layouty.get(i-1).floatValue();
        }   
    }

    public List<Float> getGridLayoutX() {
        return layoutx;
    }

    public List<Float> getGridLayoutY() {
        return layouty;
    }

    public IGridElement getGridElement(int x, int y) {
        return gridelements[x][y];
    }

    public double[][] getFactors() {
        return factor;
    }

    public double getVolume() {
        return volume;
    }

    public float[] getRelativeGridLayoutX() {
        return relativeGridLayoutX;
    }

    public float[] getRelativeGridLayoutY() {
        return relativeGridLayoutY;
    }

}
