package main.core.data.dt.primitives;

import main.core.data.IGrid;
import main.util.WufiUtils;
import translation.Translation;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;

/*
 * @author Pankratz
 * Date: 07.12.2005
 * Time: 13:27:47
 */

/**
 * $Id: Utils.java,v 1.1 2006/09/22 13:25:25 fpankratz Exp $
 */
public class Utils {
    private static SimpleDateFormat sdf1 = WufiUtils.getSimpleDateFormat("dd.MM.yyyy_h.mm");
    private static SimpleDateFormat sdf2 = WufiUtils.getSimpleDateFormat("dd.MM.yyyy_hh.mm");

    public static long getDate(double d) {
        long l = Math.round(d);
        return l * 1000;
    }

    public static Date getDate(String s) {
        //String s1 = s.substring(0,s.indexOf("_"));
        //TODO testen
        try {
            return sdf1.parse(s);
        } catch (ParseException e) {
            try {
                return sdf2.parse(s);
            } catch (ParseException e1) {
                throw new IllegalArgumentException(Translation.translate("util.utils.date.notparsable"));
            }

        }
        //return new Date(g.getTimeInMillis());
    }

    public static Set<MyPoint> getPositions(boolean[][] selected) {
        Set<MyPoint> points = new HashSet<MyPoint>();
        int volume1 = 0;
        for (int i = 0; i < selected.length; i++) {
            for (int j = 0; j < selected[i].length; j++) {
                if (selected[i][j]) {
                    volume1++;
                    if (!isIn(i, j, points)) {
                        MyPoint p = createPoint(i, j, selected.length - 1, selected[i].length - 1, selected, points);
                        points.add(p);
                    }
                }

            }
        }

        int volume2 = 0;
        for (MyPoint p : points) {
            volume2 += p.getVolume();
            for (MyPoint p2 : points) {
                if (p.intersects(p2)) {
                    if (!p.equals(p2)) {
                        throw new ArithmeticException(Translation.translate("util.utils.grid.pointsoverlap"));
                    }
                }
            }
        }
        if (volume1 != volume2) {
            throw new ArithmeticException(Translation.translate("util.utils.grid.volumediffers"));
        }
        return points;
    }

    private static MyPoint createPoint(int x, int y, int maxx, int maxy, boolean[][] selected, Set<MyPoint> points) {
        int xe = maxx;
        int ye = maxy;
        for (int i = x; i < selected.length; i++) {
            int ymax = maxy;
            if (!selected[i][y]) {
                xe = i - 1;
                break;
            } else {
                if (isIn(i, y, points)) {
                    xe = i - 1;
                    break;
                }
            }
            for (int j = y; j < selected[i].length; j++) {
                if (!selected[i][j]) {
                    ymax = j - 1;
                    break;
                } else {
                    if (isIn(i, j, points)) {
                        ymax = j - 1;
                        break;
                    }
                }
            }
            ye = Math.min(ye, ymax);
        }
        return new MyPoint(x, y, xe, ye);
    }

    private static boolean isIn(int x, int y, Set<MyPoint> points) {
        for (MyPoint p : points) {
            if (p.contains(x, y)) return true;
        }
        return false;
    }

    public static boolean[][] selectWholeGrid(IGrid grid){
        boolean[][] selected = new boolean[grid.getGridLayoutX().size()][grid.getGridLayoutY().size()];
        for (int i = 0; i < selected.length; i++) {
            for (int j = 0; j < selected[i].length; j++) {
                if (grid.getGridElement(i, j).getMaterial() != null) {
                    selected[i][j] = true;
                } else
                    selected[i][j] = false;
            }
        }
        return selected;
    }

    public static Set<MyPoint> selectWholeGridPositions(IGrid grid){
        boolean[][] selected = selectWholeGrid(grid);        
        Set<MyPoint> selectAll = getPositions(selected);
        return selectAll;
    }
}
