package main.core.data.et;

import main.core.data.dt.*;
import main.core.data.dt.primitives.MyPoint;
import main.core.data.file.IDataInput;
import main.core.data.*;

import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.Date;
import java.util.Set;

/*
 * @author Pankratz
 * Date: 16.11.2005
 * Time: 14:53:21
 */

/**
 * $Id: Wufi2DMovie.java,v 1.1 2006/09/22 13:25:25 fpankratz Exp $
 */
public class Wufi2DMovie implements IWufiData {
    private IWufiData data;
    private String name;
    private String filename;

    public Wufi2DMovie(IDataInput di,String name,String filename) throws IOException {
        this.name = name;
        this.filename = filename;
        Header header = readHeader(di);
        if (!header.getInfo().equals("$w2dflow$")) throw new IllegalArgumentException("Verion not supported");
        switch(header.getVersion()){
            case 3:
            case 4:
                    data = new Wufi2DMovieVer3(di,header);
                    break;
            default:
                throw new IllegalArgumentException("Verion not supported");
        }
    }

    /**
     * Size of Header 40 Byte
     *
     * @param di
     * @return
     */
    private Header readHeader(IDataInput di) throws IOException {
        String info;
        short version;
        int gipos;
        int optpos;
        int rgbpos;
        int cipos;
        int gridpos;
        int datapos;
        int ncts;

        info = di.readPascalString(9);

        version = di.readShort();
        gipos = di.readInt();
        optpos = di.readInt();
        rgbpos = di.readInt();
        cipos = di.readInt();
        gridpos = di.readInt();
        datapos = di.readInt();
        ncts = di.readInt();
        Header header = new Header(info, version, gipos, optpos, rgbpos, cipos, gridpos, datapos, ncts);
        return header;

    }

    public synchronized IOptions getOptions(){
        return data.getOptions();
    }

    public synchronized IGrid getGrid(){
        return data.getGrid();
    }

    public synchronized ICalculationInformation getCalcInfo(){
        return data.getCalcInfo();
    }

    public synchronized List<IMaterial> getMaterials() {
        return data.getMaterials();
    }

    public synchronized ICalculationResult getCalculationResults(Enum resultOption, Set<MyPoint> positions, Date startdate, Date enddate) {
        return data.getCalculationResults(resultOption,positions, startdate, enddate);
    }

    public synchronized Date getResultElements(Enum resultOption, long step,MyPoint position, Map<Enum,float[][]> result)  throws IOException{
        return data.getResultElements(resultOption,step,position,result);
    }

    public synchronized String getName() {
        return name;
    }

    public synchronized String getFileName() {
        return filename;
    }

    public boolean hasData() {
        return data.hasData();
    }
}
