package main.core.data.et;

import main.core.data.*;
import main.core.data.constants.CoreParameter;
import main.core.data.constants.OneArrayResults;
import main.core.data.constants.TwoArrayResults;
import main.core.data.constants.TwoArrayVektor;
import main.core.data.dt.*;
import main.core.data.dt.primitives.MinMax;
import main.core.data.dt.primitives.MyPoint;
import main.core.data.dt.primitives.Utils;
import main.core.data.file.IArrayInput;
import main.core.data.file.IDataInput;
import translation.Translation;

import java.awt.*;
import java.io.IOException;
import java.util.*;
import java.util.List;

/*
 * @author Pankratz
 * Date: 30.11.2005
 * Time: 09:21:14
 */

/**
 * $Id: Wufi2DMovieVer3.java,v 1.5 2006/12/04 14:20:39 fpankratz Exp $
 */
public class Wufi2DMovieVer3 implements IWufiData {
    private IDataInput di;
    private IOptions options;
    private ICalculationInformation calcinfo;
    private List<IMaterial> materials;
    private IGrid grid;
    private int stepCount;
    private long gridsize;
    private long datablocksize;
    private Map<Enum, Long> offset;
    private Header header;

    public Wufi2DMovieVer3(IDataInput di, Header header) throws IOException {
        this.di = di;
        this.header = header;
        getOptions_int();
        getCalcInfo_int();
    }

    private void getOptions_int() throws IOException {
        List<MinMax> ub = new ArrayList<MinMax>();
        float min;
        float max;
        for (int i = 0; i < 8; i++) {
            min = di.readFloat();
            max = di.readFloat();
            ub.add(new MinMax(min, max));

        }
        options = new Options(ub);
    }

    private void getCalcInfo_int() throws IOException {
        float min;
        float max;
        String s = di.readPascalString(16);
        Date startDate = Utils.getDate(s);
        Map<Enum, Boolean> resultOptions = new HashMap<Enum, Boolean>();

        if (di.readShort() != 0) {
            resultOptions.put(OneArrayResults.ksWC, true);
            resultOptions.put(OneArrayResults.ksWCMass, true);
        } else {
            resultOptions.put(OneArrayResults.ksWC, false);
            resultOptions.put(OneArrayResults.ksWCMass, false);
        }

        if (di.readShort() != 0) {
            resultOptions.put(OneArrayResults.ksRH, true);
        } else {
            resultOptions.put(OneArrayResults.ksRH, false);
        }

        if (di.readShort() != 0) {
            resultOptions.put(OneArrayResults.ksTeta, true);
        } else {
            resultOptions.put(OneArrayResults.ksTeta, false);
        }

        if (di.readShort() != 0) {
            resultOptions.put(OneArrayResults.ksPd, true);
        } else {
            resultOptions.put(OneArrayResults.ksPd, false);
        }

        if (di.readShort() != 0) {
            resultOptions.put(TwoArrayResults.kvFluK, true);
        } else {
            resultOptions.put(TwoArrayResults.kvFluK, false);
        }
        if (di.readShort() != 0) {
            resultOptions.put(TwoArrayResults.kvFluD, true);
        } else {
            resultOptions.put(TwoArrayResults.kvFluD, false);
        }
        if (di.readShort() != 0) {
            resultOptions.put(TwoArrayResults.kvFluT, true);
        } else {
            resultOptions.put(TwoArrayResults.kvFluT, false);
        }
        if (di.readShort() != 0) {
            resultOptions.put(TwoArrayResults.kvVXY, true);
        } else {
            resultOptions.put(TwoArrayResults.kvVXY, false);
        }

        String wndName = null;//di.readPascalString(32);

        Map<Enum, Boolean> parameter = new HashMap<Enum, Boolean>();
        for (CoreParameter cp : CoreParameter.values()) {
            parameter.put(cp, di.readBoolean());
        }

        List<MinMax> minMax = new ArrayList<MinMax>();
        for (int j = 0; j < 8; j++) {
            min = di.readFloat();
            max = di.readFloat();
            minMax.add(new MinMax(min, max));
        }

        // muss zuerst ausgelesen werden da ich sonst nicht auf die
        // groesse der Datensaetze schliessen kann
        getGridAndMaterials_int();

        // offsets and endDate
        offset = new HashMap<Enum, Long>();
        long position = di.getFilePointer();

        // Datum am Anfang der Datenbloecke
        datablocksize = 8;
        gridsize = grid.getGridLayoutX().size() * grid.getGridLayoutY().size() * 4;

        if (resultOptions.get(OneArrayResults.ksWC)) {
            offset.put(OneArrayResults.ksWC, datablocksize);
            offset.put(OneArrayResults.ksWCMass, datablocksize);
            datablocksize += gridsize;
        }
        if (resultOptions.get(OneArrayResults.ksRH)) {
            offset.put(OneArrayResults.ksRH, datablocksize);
            datablocksize += gridsize;
        }
        if (resultOptions.get(OneArrayResults.ksTeta)) {
            offset.put(OneArrayResults.ksTeta, datablocksize);
            datablocksize += gridsize;
        }
        if (resultOptions.get(OneArrayResults.ksPd)) {
            offset.put(OneArrayResults.ksPd, datablocksize);
            datablocksize += gridsize;
        }

        if (resultOptions.get(TwoArrayResults.kvFluK)) {
            offset.put(TwoArrayResults.kvFluK, datablocksize);
            datablocksize += gridsize;
            offset.put(TwoArrayVektor.valueOf(TwoArrayResults.kvFluK.name()), datablocksize);
            datablocksize += gridsize;
        }

        if (resultOptions.get(TwoArrayResults.kvFluD)) {
            offset.put(TwoArrayResults.kvFluD, datablocksize);
            datablocksize += gridsize;
            offset.put(TwoArrayVektor.valueOf(TwoArrayResults.kvFluD.name()), datablocksize);
            datablocksize += gridsize;
        }

        if (resultOptions.get(TwoArrayResults.kvFluT)) {
            offset.put(TwoArrayResults.kvFluT, datablocksize);
            datablocksize += gridsize;
            offset.put(TwoArrayVektor.valueOf(TwoArrayResults.kvFluT.name()), datablocksize);
            datablocksize += gridsize;
        }

        if (resultOptions.get(TwoArrayResults.kvVXY)) {
            offset.put(TwoArrayResults.kvVXY, datablocksize);
            datablocksize += gridsize;
            offset.put(TwoArrayVektor.valueOf(TwoArrayResults.kvVXY.name()), datablocksize);
            datablocksize += gridsize;
        }


        stepCount = 0;

        int j = header.getNcts();
        double end = 0;
        for (int i = j; i >= 1; i--) {
            long lpom = position + datablocksize * i;
            if (lpom < di.length()) {
                di.seek(lpom);
                double d2 = di.readDouble();
                if (d2 < end) {
                    break;
                } else {
                    end = d2;
                }
                stepCount = i;
            }

        }

        Date endDate = new Date(Utils.getDate(end) + startDate.getTime());
        di.seek(position);
        calcinfo = new CalculationInformation(startDate, endDate, wndName, resultOptions, parameter, minMax);
    }

    private void getGridAndMaterials_int() throws IOException {
        // auslesen gridlayout
        int pxm = di.readInt();
        int pym = di.readInt();

        List<Float> px = new ArrayList<Float>();
        for (int i = 0; i < pxm; i++) {
            px.add(new Float(di.readFloat()));
        }

        List<Float> py = new ArrayList<Float>();
        for (int i = 0; i < pym; i++) {
            py.add(new Float(di.readFloat()));
        }



        // auslesen zuordnung grid zu material
        int anz_mat = di.readInt();

        short mat_zu_grid[][] = new short[pxm][pym];
        for (int i = 0; i < pxm; i++) {
            for (int j = 0; j < pym; j++) {
                mat_zu_grid[i][j] = di.readShort();
            }
        }

        // materialien auslesen
        materials = new ArrayList<IMaterial>();
        for (int i = 0; i < anz_mat; i++) {
            materials.add(readMaterial());
        }
        IMaterial gridelements[][] = new IMaterial[pxm][pym];
        for (int i = 0; i < pxm; i++) {
            for (int j = 0; j < pym; j++) {
                if (mat_zu_grid[i][j] != 0) {
                    gridelements[i][j] = materials.get(mat_zu_grid[i][j] - 1);
                } else {
                    gridelements[i][j] = null;
                }

            }
        }
        grid = new Grid(px, py, gridelements);
    }

    private IMaterial readMaterial() throws IOException {
        String name = di.readCharString(50);
        short r = di.readShort();
        short g = di.readShort();
        short b = di.readShort();
        Color color = new Color(r, g, b);
        short nlmbx = di.readShort();
        short nlmby = di.readShort();
        short nmyx = di.readShort();
        short nmyy = di.readShort();
        short nfksx = di.readShort();
        short nfksy = di.readShort();
        short nfkrx = di.readShort();
        short nfkry = di.readShort();
        short nsor = di.readShort();
        float rho = di.readFloat();
        float w95 = di.readFloat();
        float cap = di.readFloat();
        float dlamx = di.readFloat();
        float dlamy = di.readFloat();
        float zulmd = di.readFloat();
        float por = di.readFloat();
        float wf = di.readFloat();
        float wmax = di.readFloat();
        float tetai = di.readFloat();
        float rhi = di.readFloat();

        float[][] bltabx = getMaterialTable(nlmbx);

        float[][] bltaby = getMaterialTable(nlmby);

        float[][] bmytabx = getMaterialTable(nmyx);

        float[][] bmytaby = getMaterialTable(nmyy);

        float[][] fktabsx = getMaterialTable(nfksx);
        float[][] fktabsy = getMaterialTable(nfksy);
        float[][] fktabrx = getMaterialTable(nfkrx);
        float[][] fktabry = getMaterialTable(nfkry);
        float[][] sortab = new float[nsor][3];
        for (int i = 0; i < nsor; i++) {
            sortab[i][0] = di.readFloat();
            sortab[i][1] = di.readFloat();
            sortab[i][2] = di.readFloat();
        }
        return new Material(name, color, rho, w95, cap, dlamx, dlamy, zulmd, por, wf, wmax, tetai, rhi, bltabx, bltaby, bmytabx, bmytaby, sortab, fktabsx, fktabsy, fktabrx, fktabry);
    }

    private float[][] getMaterialTable(int x) throws IOException {
        float[][] result = new float[x][2];
        for (int i = 0; i < x; i++) {
            result[i][0] = di.readFloat();
            result[i][1] = di.readFloat();
        }
        return result;
    }

    public synchronized IGrid getGrid() {
        return grid;
    }

    public synchronized IOptions getOptions() {
        return options;
    }

    public synchronized ICalculationInformation getCalcInfo() {
        return calcinfo;
    }

    public synchronized List<IMaterial> getMaterials() {
        return materials;
    }

    public synchronized ICalculationResult getCalculationResults(Enum resultOption, Set<MyPoint> positions, Date startdate, Date enddate) {
        Date[] res1 = new Date[1];
        Date[] res2 = new Date[1];
        int sps = getStepForDate(startdate, res1);
        int spe = getStepForDate(enddate, res2);
        return new CalculationResult(this, resultOption, res1[0], res2[0], sps, spe, positions, grid);
    }

    private int getStepForDate(Date d, Date[] res) {
        if (calcinfo.getStartDate().equals(d)) {
            res[0] = calcinfo.getStartDate();
            return 0;
        }
        if (calcinfo.getEndDate().equals(d)) {
            res[0] = calcinfo.getEndDate();
            return stepCount - 1;
        }
        int step = 0;
        long ds = d.getTime();
        long dstart = calcinfo.getStartDate().getTime();
        while (step < stepCount) {
            long pos = header.getDatapos() + datablocksize * step;
            try {
                di.seek(pos);
                Double endDate = di.readDouble();
                long dcurrent = dstart + Utils.getDate(endDate);
                // if the current date from the current step is greater then the date we seek
                // we return the current step - 1
                if (dcurrent > ds) return step - 1;
                res[0] = new Date(dcurrent);
                step++;
            } catch (IOException e) {
                return -1;
            }
        }
        return -1;
    }

    public synchronized Date getResultElements(Enum resultOption, long step, MyPoint position, Map<Enum, float[][]> result) throws IOException {
        if (!calcinfo.getResultOptions().get(resultOption)) throw new IllegalArgumentException(Translation.translate("data.error.resultnotavailable"));
        float[][] fa;

        long pos = header.getDatapos() + datablocksize * step;
        di.seek(pos);
        Double endDate = di.readDouble();


        // if it is a single value result
        if (resultOption.getDeclaringClass() == OneArrayResults.class) {
            fa = readResults(resultOption, pos, position);
            if (resultOption.equals(OneArrayResults.ksWCMass)) {
                for (int i = 0; i < fa.length; i++) {
                    for (int j = 0; j < fa[i].length; j++) {
                        IMaterial m = grid.getGridElement(position.getXstart() + i, position.getYstart() + j).getMaterial();
                        float f = m.getRho();
                        fa[i][j] = fa[i][j] / f;
                    }
                }
            }
            result.put(resultOption, fa);
        }
        // if it is a vector result
        else if (resultOption.getDeclaringClass() == TwoArrayResults.class) {
            fa = readResults(resultOption, pos, position);
            result.put(resultOption, fa);
            Enum e = TwoArrayVektor.valueOf(resultOption.name()); 
            fa = readResults(e, pos, position);
            result.put(e, fa);
        }

        Date d = new Date(calcinfo.getStartDate().getTime() + Utils.getDate(endDate));
        return d;
    }

    private float[][] readResults(Enum ro, long pos, MyPoint position) throws IOException {
        long pos2 = pos + offset.get(ro).longValue();
        int ysize = grid.getGridLayoutY().size();
        int xsize = grid.getGridLayoutX().size();

        di.seek(pos2);

        float[][] result = new float[position.getXLength()][position.getYLength()];
        int xstart = position.getXstart();
        int ystart = position.getYstart();

        IArrayInput ai = di.readFortranFloatArray(xsize * ysize);
        int xlength = position.getXLength();
        int ylength = position.getYLength();
        for (int i = 0; i < xlength; i++) {
            int start = (xstart + i) * ysize + ystart;
            ai.seek(start);
            for (int j = 0; j < ylength; j++) {
                if (ro.equals(OneArrayResults.ksRH) || ro.equals(OneArrayResults.ksWCMass)) {
                    result[i][j] = ai.readFloat() * 100.0f;
                } else {
                    result[i][j] = ai.readFloat();
                }

            }
        }

        return result;
    }

    public String getName() {
        // should never be used since this class is used by Wufi2DMovie
        return null;
    }

    public String getFileName() {
        // should never be used since this class is used by Wufi2DMovie
        return null;
    }

    public boolean hasData() {
        return true;
    }
}
