package main.core.data.et;

import main.core.data.*;
import main.core.data.dt.Material;
import main.core.data.dt.Grid;
import main.core.data.dt.primitives.MyPoint;
import translation.Translation;

import java.awt.*;
import java.io.IOException;
import java.util.*;
import java.util.List;

/*
 * @author Pankratz
 * Date: 20.03.2006
 * Time: 09:52:22
 */

/**
 * $Id: WufiNoFile.java,v 1.1 2006/09/22 13:25:25 fpankratz Exp $
 */
public class WufiNoFile implements IWufiData {
    private String name;
    private String filename;
    private List<IMaterial> materials;
    private IGrid grid;

    public WufiNoFile(String name, String filename) {
        this.name = name;
        this.filename = filename;
        materials = null;
        grid = null;
    }

    public WufiNoFile(Map<String, Object> data) {
        name = (String) data.get("name");
        filename = (String) data.get("filename");
        materials = new ArrayList<IMaterial>();
        List<Map<String, Object>> mats = (List<Map<String, Object>>) data.get("materials");
        for (int i = 0; i < mats.size(); i++) {
            Map<String, Object> map = mats.get(i);
            materials.add(createMaterial(map));
        }

        List<Float> layoutx = (List<Float>)data.get("layoutx");
        List<Float> layouty = (List<Float>)data.get("layouty");
        IMaterial[][] matToGrid = new IMaterial[layoutx.size()][layouty.size()];
        int[][] materialToGrid = (int[][])data.get("materialToGrid");
        for (int i = 0; i < materialToGrid.length; i++) {
            int[] ints = materialToGrid[i];
            for (int j = 0; j < ints.length; j++) {
                int mat = ints[j];
                if(mat < 0) matToGrid[i][j] = null;
                else matToGrid[i][j] = materials.get(mat);
            }
        }
        grid = new Grid(layoutx,layouty,matToGrid);
    }

    private IMaterial createMaterial(Map<String, Object> data) {
        String name = (String) data.get("name");
        Color color = (Color) data.get("color");
        float rho = (Float) data.get("rho");
        float w95 = (Float) data.get("w95");
        float cap = (Float) data.get("cap");
        float dlamx = (Float) data.get("dlamx");
        float dlamy = (Float) data.get("dlamy");
        float zulmd = (Float) data.get("zulmd");
        float por = (Float) data.get("por");
        float wf = (Float) data.get("wf");
        float wmax = (Float) data.get("wmax");
        float tetai = (Float) data.get("tetai");
        float rhi = (Float) data.get("rhi");

        float[][] bltabx = (float[][]) data.get("bltabx");
        float[][] bltaby = (float[][]) data.get("bltaby");
        float[][] bmytabx = (float[][]) data.get("bmytabx");
        float[][] bmytaby = (float[][]) data.get("bmytaby");
        float[][] sortab = (float[][]) data.get("sortab");
        float[][] fktabsx = (float[][]) data.get("fktabsx");
        float[][] fktabsy = (float[][]) data.get("fktabsy");
        float[][] fktabrx = (float[][]) data.get("fktabrx");
        float[][] fktabry = (float[][]) data.get("fktabry");

        return new Material(name, color, rho, w95, cap, dlamx, dlamy, zulmd, por, wf, wmax, tetai, rhi, bltabx, bltaby, bmytabx, bmytaby, sortab, fktabsx, fktabsy, fktabrx, fktabry);
    }

    public IOptions getOptions() {
        throw new UnsupportedOperationException(Translation.translate("data.error.noresultfile"));
    }

    public ICalculationInformation getCalcInfo() {
        throw new UnsupportedOperationException(Translation.translate("data.error.noresultfile"));
    }

    public IGrid getGrid() {
        if (grid == null) throw new UnsupportedOperationException(Translation.translate("data.error.noresultfile"));
        return grid;
    }

    public List<IMaterial> getMaterials() {
        if (materials == null) throw new UnsupportedOperationException(Translation.translate("data.error.noresultfile"));
        return materials;
    }

    public ICalculationResult getCalculationResults(Enum resultOption, Set<MyPoint> positions, Date startdate, Date enddate) {
        throw new UnsupportedOperationException(Translation.translate("data.error.noresultfile"));
    }

    public Date getResultElements(Enum resultOption, long step, MyPoint position, Map<Enum, float[][]> result) throws IOException {
        throw new UnsupportedOperationException(Translation.translate("data.error.noresultfile"));
    }

    public String getName() {
        return name;
    }

    public String getFileName() {
        return filename;
    }

    public boolean hasData() {
        return false;
    }
}
