package main.core.data.file;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;

/*
 * @author Pankratz
 * Date: 16.11.2005
 * Time: 15:28:48
 */

/**
 * $Id: LEDataInput.java,v 1.1 2006/09/22 13:25:25 fpankratz Exp $
 */
public class LEDataInput implements IDataInput {
    byte w[];

    protected RandomAccessFile r;

    /**
     * constructor
     *
     * @param f file
     * @throws java.io.IOException
     */
    public LEDataInput(File f) throws IOException {
        r = new RandomAccessFile(f, "r");
        w = new byte[8];
    }

    /**
     * constructors
     *
     * @param f name of file
     * @throws IOException
     */
    public LEDataInput(String f) throws IOException {
        r = new RandomAccessFile(f, "r");
        w = new byte[8];
    }

    public void readFully(byte b[]) throws IOException {
        r.readFully(b, 0, b.length);
    }

    public void readFully(byte b[], int off, int len) throws IOException {
        r.readFully(b, off, len);
    }

    public int skipBytes(int n) throws IOException {
        return r.skipBytes(n);
    }

    public boolean readBoolean() throws IOException {
        return r.readBoolean();
    }

    public byte readByte() throws IOException {
        return r.readByte();
    }

    public int readUnsignedByte() throws IOException {
        return r.readUnsignedByte();
    }

    public short readShort() throws IOException {
        r.readFully(w, 0, 2);
        return (short) ((w[1] & 0xff) << 8 | (w[0] & 0xff));
    }

    public int readUnsignedShort() throws IOException {
        r.readFully(w, 0, 2);
        return ((w[1] & 0xff) << 8 | (w[0] & 0xff));
    }

    public char readChar() throws IOException {
        r.readFully(w, 0, 2);
        return (char) ((w[1] & 0xff) << 8 | (w[0] & 0xff));
    }

    public int readInt() throws IOException {
        r.readFully(w, 0, 4);
        return (w[3]) << 24
                | (w[2] & 0xff) << 16
                | (w[1] & 0xff) << 8
                | (w[0] & 0xff);
    }

    public long readLong() throws IOException {
        r.readFully(w, 0, 8);
        return (long) (w[7]) << 56
                | /* long cast necessary or shift done modulo 32 */
                (long) (w[6] & 0xff) << 48
                | (long) (w[5] & 0xff) << 40
                | (long) (w[4] & 0xff) << 32
                | (long) (w[3] & 0xff) << 24
                | (long) (w[2] & 0xff) << 16
                | (long) (w[1] & 0xff) << 8
                | (long) (w[0] & 0xff);
    }

    public float readFloat() throws IOException {
        return Float.intBitsToFloat(readInt());
    }

    public double readDouble() throws IOException {
        return Double.longBitsToDouble(readLong());
    }

    public String readLine() throws IOException {
        return r.readLine();
    }

    public String readUTF() throws IOException {
        return r.readUTF();
    }

    public long readCardinal() throws IOException {
        r.readFully(w, 0, 4);
        return (long) (0x00) << 56
                | /* long cast necessary or shift done modulo 32 */
                (long) (0x00) << 48
                | (long) (0x00) << 40
                | (long) (0x00) << 32
                | (long) (w[3] & 0xff) << 24
                | (long) (w[2] & 0xff) << 16
                | (long) (w[1] & 0xff) << 8
                | (long) (w[0] & 0xff);
    }

    public void close() throws IOException {
        r.close();
    }

    public long length() throws IOException {
        return r.length();
    }

    public void seek(long pos) throws IOException {
        r.seek(pos);
    }

    public String readPascalString(int size) throws IOException {
        byte chars = r.readByte();
        //if(chars > size) throw new IOException("Error in readPascalString chars>size");
        StringBuffer b = new StringBuffer();
        for (int i = 0; (i < size) || (i < chars); i++) {
            b.append((char) r.readByte());
        }
        r.skipBytes(size - chars);
        b.trimToSize();
        return b.toString();
    }

    public String readCharString(int size) throws IOException {
        StringBuffer b = new StringBuffer();
        for (int i = 0; i < size; i++) {
            b.append((char) r.readByte());
        }
        b.trimToSize();
        return b.toString();
    }

    public long getFilePointer() throws IOException {
        return r.getFilePointer();
    }

    public IArrayInput readArray(int length) throws IOException {
        byte[] b = new byte[length];
        this.readFully(b);
        return new LEArrayInput(b);
    }

    public float[][] readFortranFloatArray(int dimx, int dimy) throws IOException {
        float[][] result = new float[dimx][dimy];
        byte[] ba = new byte[dimx*dimy*4];
        r.readFully(ba);
        IArrayInput ai = new LEArrayInput(ba);
        for (int i = 0; i < dimx; i++) {
            for (int j = 0; j < dimy; j++) {
                result[i][j] = ai.readFloat();
            }
        }
        return result;
    }

    public IArrayInput readFortranFloatArray(int count) throws IOException {
        byte[] ba = new byte[count*4];
        r.readFully(ba);
        IArrayInput ai = new LEArrayInput(ba);
        return ai;
    }
}
